% This program computes responses to an impulse at t = 10 in the innovation in the 
% autoregressive process for productivity in the stochastic, discrete-time version 
% of the Cass-Koopmans economy. The Blanchard and Kahn method is used to generate 
% times series for the endogenous variables along the transition.
%
% The single Figure produced displays the responses for the endogenous variables
%
% Structural parameter values can be changed in the first block of instructions
% [Copyright: J.Ruiz]

clear

%  structural parameters

A=1.0;              %    technology parameter
beta= 0.95;      	%    discount factor
sigma=3.0;      	%    inverse of the intertemporal elasticity of substitution 
%                        (relative risk aversion)
n=0.01;             %    population growth rate
delta=0.1;          %    depreciation rate
alpha=0.35;         %    capital share
tauc=0.12;      	%    consumption tax rate
tauy=0.2;           %    income tax rate
rho=0.95;           %    autoregressive parameter
sigmae=0.01;     	%    standard desviation of innovation


%  steady state

thetass=1;
kss=( alpha*thetass*A*(1-tauy)/(((1+n)/beta)-(1-delta)) )^(1/(1-alpha));
yss=A*thetass*kss^alpha;
css=(kss/(1+tauc))*((1-tauy)*(yss/kss)-(n+delta));
iss=(n+delta)*kss;
rss=alpha*(yss/kss)-delta;
gss=tauy*yss+tauc*css;
gyss=gss/yss;
ydss=(1-tauy)*yss;


%  initial conditions

nobs=100;
k0=kss;
theta0=thetass;

eps=randn(nobs+100,1)*sigmae;
    
    % impulse in t=10
    eps=eps(101:nobs+100)*0.0;   		 %     innovaciones
    eps(10)=sigmae;

%  Computing A, B, C, D, F

MA=[(1-alpha)*(((1+n)/beta)-1+delta)         ((1+n)/beta)*sigma;
          (1+n)                                                      0 ];

MB=[ 0                         ((1+n)/beta)*sigma;
           (1+n)/beta         -(1+tauc)*css/kss ];

MC=[ rho*(((1+n)/beta)-1+delta);
           (((1+n)/beta)-1+delta)/alpha];

MD=inv(MA)*MB;

MF=inv(MA)*MC;


%  Computing   m ,  Q

  [kvec,keig]=eig(MD);
  
  keig2=diag(keig);

  [keig3,kord]=sort(abs(keig2));

  if keig3(1)>1
     'error - no solution'
  end

  if keig3(2)<1
     'error - multiple solutions'
  end

  keig4=keig2(kord);

  Mlambda=diag(keig4);

  MM=kvec(:,kord);

  Mm=inv(MM);

  MQ=Mm*MF;


%  Computing paths for physical capital, shock and consumption

G=(MQ(1)-((rho-Mlambda(1,1))/(rho-Mlambda(2,2)))*(MQ(2)*Mm(1,2)/Mm(2,2)))/(Mm(1,1)-Mm(1,2)*Mm(2,1)/Mm(2,2));

MT=[Mlambda(1,1)        G  ;
         0                             rho];    %  transition matrix

MI=[0;1];     % innovations effects matrix

MC=[-Mm(2,1)/Mm(2,2)        (MQ(2)/Mm(2,2))/(rho-Mlambda(2,2))];     %   control matrix


%%%%%%%%%%%%


estad0=[log(k0/kss); log(theta0/thetass)];


ct=[];
estad=[];

for t=1:nobs
    
   cte=MC*estad0;
   ct=[ct; cte];
   estade=MT*estad0+MI*eps(t);
   estad=[estad; estade'];
   estad0=estade;
   
end

c=[];
k=[];
theta=[];
y=[];
i=[];  
r=[];
g=[];
gy=[];
yd=[];
ferror=[];

thetaant=theta0;
kant=k0;

for t=1:nobs

    ce=css*exp(ct(t));
    c=[c; ce];

    ke=kant;
    k=[k; ke];
    kant=kss*exp(estad(t,1));

    thetae=thetaant;
    theta=[theta; thetae];
    thetaant=thetass*exp(estad(t,2));

    ye=A*thetae*ke^alpha;
    y=[y; ye];

    re=alpha*(ye/ke)-delta;
    r=[r ; re];
    
    ge=tauy*ye+tauc*ce;
    g=[g; ge];
    
    gye=ge/ye;
    gy=[gy; gye];
    
    yde=(1-tauy)*ye;
    yd=[yd; yde];
    
end


    
for t=1:nobs-1

      ie=(1+n)*k(t+1)-(1-delta)*k(t);
      i=[i  ; ie] ;
      
      ferrore=(c(t+1)^(-sigma))*((1-tauy)*alpha*A*theta(t+1)*...
          (k(t+1)^(alpha-1))+(1-delta))-((1+n)/beta)*(c(t)^(-sigma));
      ferror=[ferror; ferrore];
end

    

% graphs
vcss=ones(size(c,1),1)*css;
vyss=ones(size(c,1),1)*yss;
vkss=ones(size(c,1),1)*kss;
vrss=ones(size(c,1),1)*rss;
vthss=ones(size(c,1),1)*thetass;
viss=ones(size(c,1)-1,1)*iss;
vgss=ones(size(c,1),1)*gss;
vgyss=ones(size(c,1),1)*gyss;
vydss=ones(size(c,1),1)*ydss;


figure;
hold on

subplot(4,2,1)
plot([c,vcss]);
title('consumtion');
xlabel('time');

subplot(4,2,2)
plot([y,vyss]);
title('output');
xlabel('time');

subplot(4,2,3)
plot([k,vkss]);
title('capital stock');
xlabel('time');

subplot(4,2,4)
plot([i,viss]);
title('investment');
xlabel('time');

subplot(4,2,5)
plot([g,vgss]);
title('public revenues');
xlabel('time');

subplot(4,2,6)
plot([gy,vgyss]);
title('public revenues to output ratio');
xlabel('time');

subplot(4,2,7)
plot([yd,vydss]);
title('disposable income');
xlabel('time');


hold off