% The Blanchard-Kahn method is used to compute a numerical solution to the discrete-time 
% version of the Cass-Koopmans economy subject to productivity shocks. A single realization 
% is obtained and graphs for the simulated series for the main variables in the model are 
% displayed.
%
% Structural parameters can be changed in the first block of instructions
%
% [Copyright: J.Ruiz]

clear

%  structural parameters 

nobs=100;            %    Number of observations

A=1.0;              	%    Tecnological parameter 
beta= 0.95;      	   %    Time discount factor
sigma=3.0;      	   %    Inverse of intertemporal elasticity of consumption
%                         (aversion relativa al riesgo)
n=0.01;            	%    Population growth
delta=0.1;        	%    Depreciation rate of capital
alpha=0.35;    	   %    Output elasticity of capital stock
tauc=0.12;      	   %    Consumption tax rate
tauy=0.2;          	%    Income tax rate
rho=0.95;         	%    Autoregressive parameter for the technology shock
sigmae=0.01;	      %    Standard deviation for innovation in technology autoregressive process	The Blanchard-Kahn method is used to compute a numerical solution to the discrete-time version of the Cass-Koopmans economy subject to productivity shocks. A single realization is obtained and graphs for the simulated series for the main variables in the model are displayed.



%  steady state 

thetass=1;
kss=( alpha*thetass*A*(1-tauy)/(((1+n)/beta)-(1-delta)) )^(1/(1-alpha));
yss=A*thetass*kss^alpha;
css=(kss/(1+tauc))*((1-tauy)*(yss/kss)-(n+delta));
iss=(n+delta)*kss;
rss=alpha*(yss/kss)-delta;


%  initial conditions

k0=kss;
theta0=thetass;



%  Computing matrices A, B, C, D, F

MA=[(1-alpha)*(((1+n)/beta)-1+delta)         ((1+n)/beta)*sigma;
          (1+n)                                                      0 ];

MB=[ 0                         ((1+n)/beta)*sigma;
           (1+n)/beta         -(1+tauc)*css/kss ];

MC=[ rho*(((1+n)/beta)-1+delta);
           (((1+n)/beta)-1+delta)/alpha];

MD=inv(MA)*MB;

MF=inv(MA)*MC;


%  Computing matrices m ,  Q

  [kvec,keig]=eig(MD);
  
  keig2=diag(keig);

  [keig3,kord]=sort(abs(keig2));

  if keig3(1)>1
     'error - no solution'
  end

  if keig3(2)<1
     'error - multiple solutions'
  end

  keig4=keig2(kord);

  Mlambda=diag(keig4);

  MM=kvec(:,kord);

  Mm=inv(MM);

  MQ=Mm*MF;

  
%  Computing trajectories for capital stock, technology shock and consumption 
  
G=(MQ(1)-((rho-Mlambda(1,1))/(rho-Mlambda(2,2)))*(MQ(2)*Mm(1,2)/Mm(2,2)))/(Mm(1,1)-Mm(1,2)*Mm(2,1)/Mm(2,2));

MT=[Mlambda(1,1)        G  ;
   0                             rho];    %  State transition matrix

MI=[0;1];     %   Matrix of effects of the innovation on the states

MC=[-Mm(2,1)/Mm(2,2)        (MQ(2)/Mm(2,2))/(rho-Mlambda(2,2))];     %   Control matrix

eps=randn(nobs+100,1)*sigmae;

eps=eps(101:nobs+100);   		 %     Innovations


estad0=[log(k0/kss); log(theta0/thetass)];


ct=[];
estad=[];

for t=1:nobs

   cte=MC*estad0;
   ct=[ct; cte];
   estade=MT*estad0+MI*eps(t);
   estad=[estad; estade'];
   estad0=estade;
   
end

c=[];
k=[];
theta=[];
y=[];
i=[];  
r=[];

thetaant=theta0;
kant=k0;

for t=1:nobs

    ce=css*exp(ct(t));
    c=[c; ce];

    ke=kant;
    k=[k; ke];
    kant=kss*exp(estad(t,1));

    thetae=thetaant;
    theta=[theta; thetae];
    thetaant=thetass*exp(estad(t,2));

    ye=A*thetae*ke^alpha;
    y=[y; ye];

    re=alpha*(ye/ke)-delta;
    r=[r ; re];

end


for t=1:nobs-1

      ie=(1+n)*k(t+1)-(1-delta)*k(t);
      i=[i  ; ie] ;

end

    

% Figure

vcss=ones(size(c,1),1)*css;
vyss=ones(size(c,1),1)*yss;
vkss=ones(size(c,1),1)*kss;
vrss=ones(size(c,1),1)*rss;
vthss=ones(size(c,1),1)*thetass;
viss=ones(size(c,1)-1,1)*iss;

figure;
hold on

subplot(2,2,1)
plot([c,vcss]);
title('consumption')
xlabel('periods')

subplot(2,2,2)
plot([y,vyss]);
title('output')
xlabel('periods')

subplot(2,2,3)
plot([k,vkss]);
title('capital stock')
xlabel('periods')

subplot(2,2,4)
plot([i,viss]);
title('investment')
xlabel('periods')

hold off
