% This program computes a single realization of the discrete-time version of the 
% Cass-Koopmans growth model under uncertainty, using SIMS method and a linear 
% approximation. This program calls function ss_sims.m, to solve for the steady-state 
% as well as to compute the transition matrix in the linear approximation to the first 
% order conditions for the optimization problem.
%
% The single Figure displays the time series paths for the endogenous variables
% 
% Structural parameter values can be changed in the first block of instructions.
%
% [Copyright: J. Ruiz]

clear
nobs=500;            %    number of observations

%  structural parameters

A=1.0;               %    technology level
beta= 0.95;      	   %    discount factor
sigma=3.0;      	   %    inverse of the intertemporal elasticity of substitution (relative risk aversion)
n=0.01;            	%    population growth rate
delta=0.1;        	%    depreciation rate
alpha=0.35;          %    capital share
tauc=0.12;      	   %    consumption tax rate
tauy=0.2;          	%    income tax rate
rho=0.95;         	%    autoregressive parameter
sigmae=0.005;     	%    standard desviation of productivity innovation


%  steady state

thetass0=1; lthetss0=log(thetass0);
kss0=( alpha*thetass0*A*(1-tauy)/(((1+n)/beta)-(1-delta)) )^(1/(1-alpha));
yss0=A*thetass0*kss0^alpha;
css0=(kss0/(1+tauc))*((1-tauy)*(yss0/kss0)-(n+delta));
iss0=(n+delta)*kss0;
rss0=alpha*(yss0/kss0)-delta;
gss0=tauy*yss0+tauc*css0;
gyss0=gss0/yss0;
ydss0=(1-tauy)*yss0;


% an alternative approach to computing the steady state
    param=[A beta sigma n delta alpha tauc tauy rho sigmae];
    x0=[kss0 kss0 css0 css0 lthetss0 lthetss0 0 0]';
    x=fsolve('ss_sims',x0,optimset,param);
    kss=x(1);
    css=x(3);
    thetass=exp(x(5));lthetass=x(5);
    ferrss=x(8);
    innss=x(7);
    yss=A*thetass*kss^alpha; 
    iss=(n+delta)*kss; 
    rss=alpha*(yss/kss)-delta;
    uss=(css^(1-sigma)-1)/(1-sigma);      
    gss=tauy*yss+tauc*css; 
    gyss=gss/yss; 
    ydss=(1-tauy)*yss;
    
    J=jacobian('ss_sims',x,param); % transition matrix
    MA=[J(1,1) J(1,3) J(1,5);
        J(2,1) J(2,3) J(2,5);
        J(3,1) J(3,3) J(3,5)];
    MB=[J(1,2) J(1,4) J(1,6);
        J(2,2) J(2,4) J(2,6);
        J(3,2) J(3,4) J(3,6)];
    Meps=-[J(1,7);
          J(2,7);
          J(3,7)];
    Minn=-[J(1,8);
          J(2,8);
          J(3,8)];
    MG=-inv(MA)*MB;
    
    [V,D]=eig(MG');

    ZI=[];
    ZR=[];
    AVALR=[];AVALI=[];
    for j=1:length(V)
        ZR=[ZR,real(V(:,j))];
        ZI=[ZI,imag(V(:,j))];
        AVALR=[AVALR;real(D(j,j))];
        AVALI=[AVALI;imag(D(j,j))];
    end

    ZZ=[];
    contr=0.0;
for j=1:length(V)
    if abs(AVALI(j))>0.000001
       if abs((AVALR(j)^2+AVALI(j)^2)^(0.5))>=1/beta
       
	     if contr~=abs((AVALR(j)^2+AVALI(j)^2)^(0.5))
                         ZZ=[ZZ ZR(:,j) ZI(:,j)];
	     end
	     contr=abs((AVALR(j)^2+AVALI(j)^2)^(0.5));             
       end
       elseif abs(AVALR(j))>=1/beta
           ZZ=[ZZ,ZR(:,j)];
    end
end


ZZ=ZZ';

   
%  initial conditions

k0=kss;         				% capital stock 
theta0=thetass; 				% shock 

eps=randn(nobs+100,1)*sigmae;

eps=eps(101:nobs+100); 		% innovations

c0=css+(-1/ZZ(2))*(ZZ(1)*(k0-kss)+ZZ(3)*(log(theta0)));
c=[];c=[c; c0];
k=[];k=[k; k0];
theta=[];theta=[theta; theta0];
y=[];y0=theta0*A*(k0^alpha);y=[y;y0];
i=[];  
r=[];r0=alpha*(y0/k0)-delta; r=[r;r0];
g=[];g0=tauy*y0+tauc*c0; g=[g;g0];
gy=[]; gy0=g0/y0; gy=[gy;gy0];
cy=[]; cy0=c0/y0; cy=[cy;cy0];
iy=[];
yd=[]; yd0=(1-tauy)*y0; yd=[yd;yd0];
ferror=[];


for t=1:nobs

    k1=(1/(1+n))*((1-tauy)*theta0*A*(k0^alpha)+(1-delta)*k0-...
        (1+tauc)*c0);
    theta1=exp(rho*log(theta0)+eps(t));
    c1=css+(-1/ZZ(2))*(ZZ(1)*(k0-kss)+ZZ(3)*(log(theta0)));
    y1=theta1*A*(k1^alpha);y=[y;y1];
    i1=(1+n)*k1-(1-delta)*k0;i=[i;i1];
    iy1=i1/y1; iy=[iy;iy1];
    ferror1=(c1^(-sigma))*((1-tauy)*alpha*A*theta1*...
          (k1^(alpha-1))+(1-delta))-((1+n)/beta)*(c0^(-sigma));
    ferror=[ferror;ferror1];
        
    r1=alpha*(y1/k1)-delta; r=[r;r1];
    g1=tauy*y1+tauc*c1; g=[g;g1];
    gy1=g1/y1; gy=[gy;gy1];
    cy1=c1/y1; cy=[cy;cy1];
    yd1=(1-tauy)*y1; yd=[yd;yd1];
    k0=k1;c0=c1;theta0=theta1;
    k=[k;k0];c=[c;c0];theta=[theta;theta0];
    
end
    

    
% graphs
vcss=ones(size(c,1),1)*css;
vyss=ones(size(c,1),1)*yss;
vkss=ones(size(c,1),1)*kss;
vrss=ones(size(c,1),1)*rss;
vthss=ones(size(c,1),1)*thetass;
viss=ones(size(c,1)-1,1)*iss;
vgss=ones(size(c,1),1)*gss;
vgyss=ones(size(c,1),1)*gyss;
vydss=ones(size(c,1),1)*ydss;


figure;
hold on

subplot(4,2,1)
plot([c,vcss]);
title('consumption');
xlabel('time');

subplot(4,2,2)
plot([y,vyss]);
title('output');
xlabel('time');

subplot(4,2,3)
plot([k,vkss]);
title('capital stock');
xlabel('time');

subplot(4,2,4)
plot([i,viss]);
title('investment');
xlabel('time');

subplot(4,2,5)
plot([g,vgss]);
title('public revenues');
xlabel('time');

subplot(4,2,6)
plot([gy,vgyss]);
title('public revenues to output ratio');
xlabel('time');

subplot(4,2,7)
plot([yd,vydss]);
title('disposable income');
xlabel('time');


hold off