% This program computes a numerical solution to the deterministic version 
% of the optimal Growth model using the Collocation method with 
% Chebyshev polynomials

%Parameters
alpha=0.36;     % output elasticity of capital
beta=0.96;      % time discount factor
sigma=1.5;      % relative risk aversion
delta=0.1;      % depreciation rate
n=0;            % population growth 
A=1;            % Productivity parameter
%%%%%%%%%%%%%%%%%
d=3;            % number of basis function used

kss=(alpha*A/(((1+n)/beta)-1+delta))^(1/(1-alpha)); %steady state stock of capital
css=A*kss^alpha-(n+delta)*kss;  % steady state consumption
lambda=0.4;                     % width of interval for k. This can be changed, as desired
kmin=kss*(1-lambda);
kmax=kss*(1+lambda);
%%%%%%%%%%%%%%%%%%%%%%%

%  Initial values for mu-coefficients

mu0=zeros(d-1,1);mu0(1)=css; mu0(2)=0.1*kss/css;
for j=1:1 

    %%%%%%%%%%%%%%%%%%
    % Chebyshev nodes 
    kp=[];
    for i=1:d
        ki=((kmax+kmin)/2)+((kmax-kmin)/2)*cos((d-i+0.5)*pi/d);
        kp=[kp;ki];
    end
    %%%%%%%%%%%%%%%%%
    % Computing the values of the mu-coefficients by solving the system of
    % equations defined by setting to zero the value of the Residual
    % functional at each of the Chebychev nodes.
    param=[alpha,beta,sigma,delta,n,A,kmin,kmax];
    mu0=[mu0;0];
    mu=fsolve('res',mu0,optimset,param,kp);
    mu0=mu;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % We approximate the optimal decision rule on a grid of values for the 
    % state variable in order to characterize the decision rule in detail.
    % If the residual functional is above the specified precision bound at 
    % some point of the grid, the number of basis functions is increased 
    % and the algorithm applied again.
   
    gridpoints=101;
    kpp=[];
    if delta==1 & n==0 & sigma==1
        c_exact=[];
    end
    c_approx=[];
    residual=[];
    for i=1:gridpoints
        kpp1=min(kp)+(i-1)*(max(kp)-min(kp))/100;
        kpp=[kpp;kpp1];
        if delta==1 & n==0 & sigma==1
            c_e=(1-alpha*beta)*A*kpp1^alpha;
            c_exact=[c_exact;c_e];
        end
        c_a=feval('Cd',kpp1,mu,kmin,kmax);
        c_approx=[c_approx;c_a];
        k1=(1/(1+n))*(A*(kpp1)^alpha+(1-delta)*kpp1-...
            feval('Cd',kpp1,mu,kmin,kmax));
        resid=(beta/(1+n))*(feval('Cd',kpp1,mu,kmin,kmax))^sigma*...
            (alpha*A*k1^(alpha-1)+1-delta)*...
            (feval('Cd',k1,mu,kmin,kmax))^(-sigma)-1;
        residual=[residual;resid];
    end
    if max(residual)<0.0000001;
        break
    end
    if max(residual)>0.0000001;
        d=d+1;
    end
    
end
   
display ('Number of basis functions')
d

    if delta==1 & n==0 & sigma==1
          plot(c_approx,c_exact,'r.')
    end
      
%%%%%%%%%%%%%%%%%%%%
% Time series simulation

T=50;
k0=1.01*kss;  % The economy starts with a stock of capital 10% above its 
              % steady-state level
C=[];
K=[]; K=[K;k0];

for t=1:T
    Ct=feval('Cd',k0,mu,kmin,kmax);
    C=[C;Ct];
    Kt=(1/(1+n))*(A*(k0)^alpha+(1-delta)*k0-Ct);
    k0=Kt;
    K=[K;k0];
end
        
    
    figure;
    hold on
    
    subplot(2,2,1)
    plot(kpp,c_approx);
    title('policy function approximated');
    xlabel('stock of capital');
    ylabel('cosumption');
    
    subplot(2,2,2)
    plot(kpp,residual);
    title('Euler condition');
    xlabel('stock of capital');
    ylabel('Euler condition');
    
    subplot(2,2,3)
    plot([C,ones(length(C),1)*css]);
    title('Consumption');
    xlabel('time');
    
    subplot(2,2,4)
    plot([K,ones(length(K),1)*kss]);
    title('Stock of capital');
    xlabel('time');
    
    hold off

    