% Simulating the stochastic Ramsey, Cass, Koopmans model using the Galerkin's 
% method of order 3 with a complete set of Chebychev polynomials

% @Copyright J. Ruiz 2007

% Parameters
alpha=0.36;     % output elasticity of capital
beta=0.96;      % time discount factor
sigma=1;      % relative risk aversion
delta=1;      % depreciation rate
n=0;            % population growth rate
A=1;            % Productivity parameter
rho=0.9;        % persistence parameter of stochastic process for structural shock 
sigmae=0.001;   % standard deviation structural innovation
%%%%%%%%%%%%%%%%%

kss=(alpha*A/(((1+n)/beta)-1+delta))^(1/(1-alpha)); %steady state stock of capital
css=A*kss^alpha-(n+delta)*kss;  %steady state consumption

%%%%%%%%%%%%%%%%%%%%%%%%
% interval for k:
lambda=0.4;    
kmin=kss*(1-lambda);
kmax=kss*(1+lambda);
% interval for log(theta):
lthemin=-(2/sqrt(1-rho^2))*sigmae;
lthemax=(2/sqrt(1-rho^2))*sigmae;
%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%
dk=3;           % number of basis function used for the state variable: capital
dz=3;           % number of basis function used for the state variable: productivity shock
d=6;        % global number of basis function used
%%%%%%%%%%%%%%%%%%%%%%%

    %%%%%%%%%%%%%%%%%%
    % Chebyshev nodes    
    % Choose the number of points of k and z in the grid
    nodesk = 6
    nodesz = 6
    kp=[];
    for i=1:nodesk
        ki=((kmax+kmin)/2)+((kmax-kmin)/2)*cos((nodesk-i+0.5)*pi/nodesk);
        kp=[kp;ki];
    end
    lthep=[];
    for i=1:nodesz
        ltheti=((lthemax+lthemin)/2)+((lthemax-lthemin)/2)*cos((nodesz-i+0.5)*pi/nodesz);
        lthep=[lthep;ltheti];
    end
    thep=exp(lthep);
    %%%%%%%%%%%%%%%%%
    
%%%%%%%%%%%%%%%%%%%%%%%
% abscissae and weights for an m-point Gauss-Hermite quadrature
m=10;
    vv=zeros(101,1);
    for i=1:101
        v0=-3+6*(i-1)/100;
        vv(i)=fsolve('root_h',v0,optimset,m);
    end
    vv=sort(vv);
    v=[];v=vv(1);
    for i=1:100
        if vv(i+1)-vv(i)>0.001
            v=[v;vv(i+1)];
        end
    end

P=[];
for i=1:m
    h=feval('herm',v(i),m-1);
    P=[P,h];
end
indep=zeros(m,1); indep(1)=pi^0.5;
omega=inv(P)*indep;
%%%%%%%%%%%%%%%%%%%%%%%%

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Computing the values of the mu-coefficients by solving the system of
    % equations defined by setting to zero the value of the Residual
    % functional at each of the Chebychev nodes.
    mu0=zeros(d,1);mu0(1)=1.37; mu0(2)=0.26; mu0(3)=0.66;mu0(4)=-0.01;
    m0(5)=0.12; m0(6)=0.05;
    if delta==1 & n==0 & sigma==1
    mu0(1)=0.3415; mu0(2)=0.0663; mu0(3)=0.4902; mu0(4)=-0.0044;
    mu0(5)=-0.0498; mu0(6)=0.0732; 
    end
    param=[alpha,beta,sigma,delta,n,A,rho,sigmae,kmin,kmax];
    %mu0=[mu0;0];
    mu=fsolve('res_s_3g',mu0,optimset,param,kp,lthep,v,omega);
    mu0=mu;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % We approximate the optimal decision rule on a grid of values for the 
    % state variable in order to characterize the decision rule in detail.    
    gridpoints=101;
    kpp=[];lthepp=[];
    for i=1:gridpoints
        kpp1=min(kp)+(i-1)*(max(kp)-min(kp))/100;
        kpp=[kpp;kpp1];
        lthepp1=min(lthep)+(i-1)*(max(lthep)-min(lthep))/100;
        lthepp=[lthepp;lthepp1];
    end
    z=tanh(lthepp);
        
    c_approx=zeros(gridpoints,gridpoints);
    residual=zeros(gridpoints,gridpoints);
    for i=1:gridpoints
        for j=1:gridpoints
            c_approx(i,j)=feval('Cds_3',kpp(i),lthepp(j),mu,kmin,kmax);
            k1=(1/(1+n))*(A*(((1+z(j))/(1-z(j)))^0.5)*(kpp(i))^alpha+(1-delta)*kpp(i)-...
                feval('Cds_3',kpp(i),lthepp(j),mu,kmin,kmax));
            sres=0;
            for M=1:m
                z1=tanh(rho*atanh(z(j))+(2^0.5)*sigmae*v(M));
                lthe1=atanh(z1);
                res1=((pi^(-0.5))*(beta/(1+n))*(feval('Cds_3',kpp(i),lthepp(j),mu,kmin,kmax))^sigma*...
                    (alpha*A*(((1+z1)/(1-z1))^0.5)*k1^(alpha-1)+1-delta)*...
                    (feval('Cds_3',k1,lthe1,mu,kmin,kmax))^(-sigma))*omega(M);
                sres=sres+res1;
            end
            residual(i,j)=sres-1;
        end
    end
    
     
%%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulation
%%%%%%%%%%%%%%%%%%%%%%%%%%
T=150;
aleatn=normrnd(0,1,T+200,1);
eps=sigmae*aleatn(201:length(aleatn),1);
k0=kss;
theta0=1;
ltheta0=log(theta0);
theta=[];theta=[theta;theta0];
C=[];
K=[]; K=[K;k0];

if delta==1 & n==0 & sigma==1
    c_exact=[];
end

for t=1:T
    Ct=feval('Cds_3',k0,ltheta0,mu,kmin,kmax);
    C=[C;Ct];
    Kt=(1/(1+n))*(A*theta0*(k0)^alpha+(1-delta)*k0-Ct);
    k0=Kt;
    K=[K;k0];
    thetat=exp(rho*log(theta0)+eps(t));
    theta0=thetat;
    ltheta0=log(theta0);
    theta=[theta;theta0];
end
       
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Graphs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    figure;
    hold on
    
    subplot(2,2,1)
    surf(lthepp,kpp,c_approx);
    title('approximated policy function');
    ylabel('stock of capital');
    xlabel('log of structural shock');
    zlabel('cosumption');
    
    subplot(2,2,2)
    surf(lthepp,kpp,residual);
    title('Euler condition');
    ylabel('stock of capital');
    xlabel('log of structural shock');
    zlabel('Euler condition');
    
    subplot(2,2,3)
    plot([C,ones(length(C),1)*css]);
    title('Consumption');
    xlabel('time');
    
    subplot(2,2,4)
    plot([K,ones(length(K),1)*kss]);
    title('Stock of capital');
    xlabel('time');
    
    hold off

