% This program computes a single realization from the discrete-time version of the Cass-Koopmans 
% growth model under uncertainty, using the linear-quadratic approximation solution method.
% The program calls function util.m, which lays out the objective function so that the numerical 
% derivatives needed to implement the solution method can be computed.
%
% The single Figure produced displays the time series paths for the endogenous variables
% Structural parameter values can be changed in the first block of instructions.
%
% [Copyright: J. Ruiz]

clear

nobs=500;            %    number of observations

%  structural parameters

A=1.0;              %    technology level
beta= 0.95;      	%    discout factor
sigma=3.0;      	%    relative risk aversion 
                    %    (inverse of the intertemporal elasticity of substitution of consumption)
n=0.01;            	%    population growth
delta=0.1;        	%    depreciation rate of physical calpital
alpha=0.35;      	%    output share of capital
tauc=0.12;      	%    consumption tax rate
tauy=0.2;          	%    income tax rate
rho=0.95;         	%    autoregressive parameter
sigmae=0.01;        %    standar desviation of innovation

param=[A beta sigma n delta alpha tauc tauy rho sigmae];

%  steady state

thetass=1;
kss=( alpha*thetass*A*(1-tauy)/(((1+n)/beta)-(1-delta)) )^(1/(1-alpha));
yss=A*thetass*kss^alpha;
css=(kss/(1+tauc))*((1-tauy)*(yss/kss)-(n+delta));
iss=(n+delta)*kss;
rss=alpha*(yss/kss)-delta;
gss=tauc*css+tauy*yss;


%  initial conditions

k0=kss;
theta0=thetass;


% linear-quadratic approximation

z=[kss,kss,log(thetass)]';
auxi=1e-8*eye(3);
MB=zeros(3,1);
for i=1:3
    MB(i)=(feval('util',z+auxi(:,i),param)-...
        feval('util',z,param))/auxi(i,i);
end

auxi=1e-5*eye(3,3);
MA=zeros(3,3);
for i=1:3
    for j=1:3
        MA(i,j)=(feval('util',z+auxi(:,i)+auxi(:,j),param)-...
            feval('util',z+auxi(:,i),param)-...
            feval('util',z+auxi(:,j),param)+...
            feval('util',z,param))/(auxi(i,i)^2);
    end
end

phi=(MA(1,1)+beta*MA(2,2))/(beta*MA(1,2));
root1=(-phi+(phi^2-4/beta)^0.5)/2;
root2=(-phi-(phi^2-4/beta)^0.5)/2;
    if abs(root1)<1
        lambda1=root1;
    else
        lambda2=root1;
    end

    if abs(root2)<1
        lambda1=root2;
    else
        lambda2=root2;
    end

    k=[];k=[k;k0];
    theta=[];theta=[theta;theta0];
    
    %     innovations
    eps=randn(nobs+100,1)*sigmae;
    eps=eps(101:nobs+100);   		

for t=1:nobs
    theta1=exp(rho*log(theta0)+eps(t));
    k1=kss+lambda1*(k0-kss)+((MA(1,3)+beta*rho*MA(2,3))/(beta*MA(1,2)*...
        (lambda2-rho)))*log(theta0);
    theta0=theta1;
    k0=k1;
    k=[k;k0];
    theta=[theta;theta0];
end

c=[];
i=[];
y=[];
g=[];
r=[];
gy=[];
cy=[];
iy=[];
yd=[];
ferror=[];

for t=1:nobs
    y1=theta(t)*A*(k(t)^alpha);
    c1=(1/(1+tauc))*(-(1+n)*k(t+1)+(1-delta)*k(t)+(1-tauy)*y1);
    i1=(1+n)*k(t+1)-(1-delta)*k(t);
    g1=tauc*c1+tauy*y1;
    r1=alpha*y1/k1-delta;
    yd1=(1-tauy)*y1;
    cy1=c1/y1;
    iy1=i1/y1;
    gy1=g1/y1;
    
    y=[y;y1];
    c=[c;c1];
    i=[i;i1];
    g=[g;g1];
    r=[r;r1];
    yd=[yd;yd1];
    cy=[cy;cy1];
    iy=[iy;iy1];
    gy=[gy;gy1];
end

for t=1:nobs-1;
    ferror1=(c(t+1)^(-sigma))*((1-tauy)*alpha*A*theta(t+1)*...
          (k(t+1)^(alpha-1))+(1-delta))-((1+n)/beta)*(c(t)^(-sigma));
    ferror=[ferror;ferror1];
end

steady_state = [css yss kss iss thetass gss rss]
display '        c         y         k         i       theta       g         r'
variance = [var(c) var(y) var(k) var(i) var(theta) var(g) var(r)]
display '        c         y         k         i       theta       g         r'
standard_deviations = [std(c) std(y) std(k) std(i) std(theta) std(g) std(r)] 
display '        c         y         k         i       theta       g         r'
coeff_variation = 100.*[std(c)/css std(y)/yss std(k)/kss std(i)/iss std(theta)/thetass std(g)/gss std(r)/rss]
display '        c         y         k         i       theta       g         r'
relative_volatility = [std(c) std(y) std(k) std(i) std(theta) std(g) std(r)]./std(y)
display '        c         y         k         i       theta       g         r'

% graphs

vcss=ones(size(c,1),1)*css;
vyss=ones(size(c,1),1)*yss;
vkss=ones(size(c,1),1)*kss;
vrss=ones(size(c,1),1)*rss;
vthss=ones(size(c,1),1)*thetass;
viss=ones(size(c,1),1)*iss;
vgss=ones(size(c,1),1)*gss;

figure;
hold on

subplot(2,2,1)
plot([c,vcss]);
title('consumption')
xlabel('periods')

subplot(2,2,2)
plot([y,vyss]);
title('output')
xlabel('periods')

subplot(2,2,3)
plot([k(1:nobs),vkss]);
title('stock of capital')
xlabel('periods')

subplot(2,2,4)
plot([i,viss]);
title('investment')
xlabel('periods')

hold off

figure;
hold on

subplot(2,2,1)
plot([theta(1:nobs),vthss]);
title('productivity shock')
xlabel('periods')

subplot(2,2,2)
plot([g,vgss]);
title('govt. expenditures')
xlabel('periods')

subplot(2,2,3)
plot([r,vrss]);
title('interest rate')
xlabel('periods')

subplot(2,2,4)
plot([i,viss]);
title('investment')
xlabel('periods')

hold off