% This program solves for the polynomial used in the den Hann-Marcet solution method to 
% approximate the expectations term in the Euler equation in the discrete-time neoclassical 
% growth model under uncertainty. 
%
% In consistency with the discussion in the text, the program allows for models differing 
% from den Haan and Marcet (1990) in the risk aversion parameter and the consumption and 
% income tax rates. All other parameters take the same values as in den Haan and Marcet paper.
% The program iterates starting from values for the parameters in the polynomial reported by 
% den Haan and Marcet for the model with no taxes, with logarithmic utility function and a 
% depreciation rate of 10%.
%
% This program calls Matlab command "fminunc" to minimize the sum of squared residuals obtained 
% when approximating the conditional expectations in the Euler equation with the polynomial 
% proposed by den Hann and Marcet. The "srmarcet.m" file is used to compute this sum of 
% squared residuals. 
%
% [Copyright: J.Ruiz]

clear

%  Final values for structural parameters of inverse of intertemporal elasticity of substitution and 
%  for tax rates. These have to be the same as in program methods.m

sigmaf=1.5;      	   %    inverse of the intertemporal elasticity of substitution (relative risk aversion)
taucf=0.0;         	   %    consumption tax rate
tauyf=0.0;             %    income tax rate

%  Initial values for structural parameters of interest are the same as in den-Haan and Marcet (1990),
%  except for the relative risk aversion (sigma), the consumption tax rate and the income 
%  tax rate.
%  DO NOT CHANGE.

A=1.0;              %    technology level
beta= 0.95;      	%    discount factor
sigma=1.0;      	%    inverse of the intertemporal elasticity of substitution (relative risk aversion)
n=0.0;            	%    population growth rate
delta=0.1;        	%    depreciation rate
alpha=0.33;         %    capital share
tauc=0.0;         	%    consumption tax rate
tauy=0.0;          	%    income tax rate
rho=0.95;         	%    autoregressive parameter
sigmae=0.005;     	%    standard desviation of innovation


%  steady state

thetass=1; 
kss=( alpha*thetass*A*(1-tauy)/(((1+n)/beta)-(1-delta)) )^(1/(1-alpha));
yss=A*thetass*kss^alpha;
css=(kss/(1+tauc))*((1-tauy)*(yss/kss)-(n+delta));
iss=(n+delta)*kss;
rss=alpha*(yss/kss)-delta;
gss=tauy*yss+tauc*css;
gyss=gss/yss;
ydss=(1-tauy)*yss;


% initial values for the parameters of the polinomial function

a0=zeros(3,1);
a0(1)=1.74;   
a0(2)=-0.56;  
a0(3)=-0.55;  
a00=a0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
T=2500;

eps=randn(T+100,1)*sigmae;
eps=eps(101:T+100); % innovations

lambda=0.5;

epsilon=0.00001;

% finding the a's for sigmaf

for ii=1:10;
    sigma=1+(sigmaf-1)*ii/10;
    ii 
 
iter=100; % number iterations maximum
for j=1:iter
    j
    k0=kss;         % capital stock initial condition 
    theta0=thetass; % shock initial condition

    c=[];c0=((beta/(1+n))*(a0(1)*(k0^a0(2))*(theta0^a0(3))))^(-1/sigma);
    k=[];k=[k;k0];
    c=[c;c0];
    theta=[]; theta=[theta;theta0];
    expect=[];
    PSI=[];PSI0=a0(1)*(k0^a0(2))*(theta0^a0(3));PSI=[PSI;PSI0];
    for t=1:T
       k1=(1/(1+n))*((1-tauy)*theta0*A*(k0^alpha)+(1-delta)*k0-(1+tauc)*c0);
       theta1=exp(rho*log(theta0)+eps(t));
       c1=((beta/(1+n))*(a0(1)*(k1^a0(2))*(theta1^a0(3))))^(-1/sigma);
       expect1=(c1^(-sigma))*((1-tauy)*alpha*A*theta1*...
        (k1^(alpha-1))+(1-delta));
       k0=k1;c0=c1;theta0=theta1;
       k=[k;k0];c=[c;c0];theta=[theta;theta0];expect=[expect;expect1];
       PSI1=a0(1)*(k1^a0(2))*(theta1^a0(3));PSI=[PSI;PSI1];
    end
    
    global vari
    vari=[expect(1:T) k(1:T) c(1:T) theta(1:T)];
    a1t=fminunc('srmarcet',a0);
    a1=lambda.*a0+(1-lambda)*a1t;
    
    if norm(a1-a0)<epsilon
        break
    else
        a0=a1;
    end
end
end

% finding the a's for taucf
for ii=1:10;
    tauc=taucf*ii/10;
    ii
        
iter=100; % maximum number of iterations 
for j=1:iter
    
    k0=kss;         % initial condition for capital stock 
    theta0=thetass; % initial condition for productivity shock 

    c=[];c0=((beta/(1+n))*(a0(1)*(k0^a0(2))*(theta0^a0(3))))^(-1/sigma);
    k=[];k=[k;k0];
    c=[c;c0];
    theta=[]; theta=[theta;theta0];
    expect=[];
    PSI=[];PSI0=a0(1)*(k0^a0(2))*(theta0^a0(3));PSI=[PSI;PSI0];
    for t=1:T
       k1=(1/(1+n))*((1-tauy)*theta0*A*(k0^alpha)+(1-delta)*k0-(1+tauc)*c0);
       theta1=exp(rho*log(theta0)+eps(t));
       c1=((beta/(1+n))*(a0(1)*(k1^a0(2))*(theta1^a0(3))))^(-1/sigma);
       expect1=(c1^(-sigma))*((1-tauy)*alpha*A*theta1*...
        (k1^(alpha-1))+(1-delta));
       k0=k1;c0=c1;theta0=theta1;
       k=[k;k0];c=[c;c0];theta=[theta;theta0];expect=[expect;expect1];
       PSI1=a0(1)*(k1^a0(2))*(theta1^a0(3));PSI=[PSI;PSI1];
    end
    
    global vari
    vari=[expect(1:T) k(1:T) c(1:T) theta(1:T)];
    a1t=fminunc('srmarcet',a0);
    a1=lambda.*a0+(1-lambda)*a1t;
    
    if norm(a1-a0)<epsilon
        break
    else
        a0=a1;
    end
end
end

% finding the a's for tauyf
for ii=1:10;
    tauy=tauyf*ii/10;
    ii
     
iter=100; % maximum number of iterations 
for j=1:iter
    
    k0=kss;         % initial condition for capital stock 
    theta0=thetass; % initial condition for productivity shock 

    c=[];c0=((beta/(1+n))*(a0(1)*(k0^a0(2))*(theta0^a0(3))))^(-1/sigma);
    k=[];k=[k;k0];
    c=[c;c0];
    theta=[]; theta=[theta;theta0];
    expect=[];
    PSI=[];PSI0=a0(1)*(k0^a0(2))*(theta0^a0(3));PSI=[PSI;PSI0];
    for t=1:T
       k1=(1/(1+n))*((1-tauy)*theta0*A*(k0^alpha)+(1-delta)*k0-(1+tauc)*c0);
       theta1=exp(rho*log(theta0)+eps(t));
       c1=((beta/(1+n))*(a0(1)*(k1^a0(2))*(theta1^a0(3))))^(-1/sigma);
       expect1=(c1^(-sigma))*((1-tauy)*alpha*A*theta1*...
        (k1^(alpha-1))+(1-delta));
       k0=k1;c0=c1;theta0=theta1;
       k=[k;k0];c=[c;c0];theta=[theta;theta0];expect=[expect;expect1];
       PSI1=a0(1)*(k1^a0(2))*(theta1^a0(3));PSI=[PSI;PSI1];
    end
    
    global vari
    vari=[expect(1:T) k(1:T) c(1:T) theta(1:T)];
    a1t=fminunc('srmarcet',a0);
    a1=lambda.*a0+(1-lambda)*a1t;
    
    if norm(a1-a0)<epsilon
        break
    else
        a0=a1;
    end
end
end

save a0.prn a0 -ascii
