% This program solves the discrete-time version of the Cass-Koopmans growth model under 
% uncertainty, using the parameterized expectations method of den Hann and Marcet. 

% Command "fminunc" is used to minimize the sum of squared residuals in the conditional 
% expectation function.. 

% Program marcet.m needs to be executed in advance to compute initial values for the 
% parameters in the polynomial used to represent the conditional expectations.
% Both programs must be in the same directory.

% [Copyright: J. Ruiz]


clear
load a0;

%  structural parameters

A=1.0;              %    technology level
beta= 0.95;      	%    discount factor
sigma=3.0;      	%    inverse of the intertemporal elasticity of substitution (relative risk aversion)
n=0.0;            	%    population growth rate
delta=0.1;        	%    depreciation rate
alpha=0.33;         %    capital share
tauc=0.10;         	%    consumption tax rate
tauy=0.20;          	%    income tax rate
rho=0.95;         	%    autoregressive parameter
sigmae=0.005;     	%    standard desviation of innovation


%  steady state

thetass=1; 
kss=( alpha*thetass*A*(1-tauy)/(((1+n)/beta)-(1-delta)) )^(1/(1-alpha));
yss=A*thetass*kss^alpha;
css=(kss/(1+tauc))*((1-tauy)*(yss/kss)-(n+delta));
iss=(n+delta)*kss;
rss=alpha*(yss/kss)-delta;
gss=tauy*yss+tauc*css;
gyss=gss/yss;
ydss=(1-tauy)*yss;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
T=5000;

eps=randn(T+100,1)*sigmae;
eps=eps(101:T+100); % innovations

lambda=0.5;

epsilon=0.00001;

    
iter=100; % number iterations maximum
for j=1:iter
    
    k0=kss;         % capital stock initial condition 
    theta0=thetass; % shock initial condition

    c=[];c0=((beta/(1+n))*(a0(1)*(k0^a0(2))*(theta0^a0(3))))^(-1/sigma);
    k=[];k=[k;k0];
    c=[c;c0];
    theta=[]; theta=[theta;theta0];
    expect=[];
    PSI=[];PSI0=a0(1)*(k0^a0(2))*(theta0^a0(3));PSI=[PSI;PSI0];
    for t=1:T
       k1=(1/(1+n))*((1-tauy)*theta0*A*(k0^alpha)+(1-delta)*k0-(1+tauc)*c0);
       theta1=exp(rho*log(theta0)+eps(t));
       c1=((beta/(1+n))*(a0(1)*(k1^a0(2))*(theta1^a0(3))))^(-1/sigma);
       expect1=(c1^(-sigma))*((1-tauy)*alpha*A*theta1*...
        (k1^(alpha-1))+(1-delta));
       k0=k1;c0=c1;theta0=theta1;
       k=[k;k0];c=[c;c0];theta=[theta;theta0];expect=[expect;expect1];
       PSI1=a0(1)*(k1^a0(2))*(theta1^a0(3));PSI=[PSI;PSI1];
    end
    
    global vari
    vari=[expect(1:T) k(1:T) c(1:T) theta(1:T)];
    a1t=fminunc('srmarcet',a0);
    a1=lambda.*a0+(1-lambda)*a1t;
    
    if norm(a1-a0)<epsilon
        break
    else
        a0=a1;
    end
end


