% This program solves solve the discrete-time version of the Cass-Koopmans growth model 
% under uncertainty, using different solution methods: linear-quadratic approximation, 
% Sims' method with either a linear or a log-linear approximation, Blanchard-Kahn method, 
% Uhlig method, and the parameterized expectations method by den Hann-Marcet. 
%
% The user must select the specific solution method by activating the corresponding line 
% in the program, and deactivating the alternative lines. This program computes the basic 
% statistics needed for a discussion along the lines of the Real Business Cycle literature. 
% Depending on the solution method chosen, the program calls functions: MLQ.m, MSIMS.m, 
% MLSIMS.m, MBK.m, MUHLIG.m, MMARCET.m.
%
% If the parameterized expectations solution method is to be used, then "marcet.m" needs to 
% be run first to compute and save numerical values for the parameters in the polynomial 
% representing the conditional expectation function, for specific numerical values for the
% structural parameters. The 'load a0' line also needs to be activated.
% 
% Structural parameter values can be changed in the first block of instructions.
%
% [Copyright: J. Ruiz]

clear

nreal=2;        		%    number of realizations
nobs=500;       		%    number of observations (data points in each realization)

%  structural parameters

A=1.0;                 %    technology level
beta= 0.95;      	   %    discount factor
sigma=1.5;      	   %    inverse of the intertemporal elasticity of substitution (relative risk aversion)
n=0.0;            	   %    population growth rate
delta=0.1;        	   %    depreciation rate
alpha=0.33;            %    capital share
rho=0.95;         	   %    autoregressive parameter
sigmae=0.005;     	   %    standard desviation of innovation

%  Set the next two parameters (consumption and income tax rates) to zero to
%  simulate the economy wihtout taxes.
tauc=0.0;      	      %    consumption tax rate
tauy=0.0;             %    income tax rate

param=[A beta sigma n delta alpha tauc tauy rho sigmae];

%  steady state

thetass=1;
kss=( alpha*thetass*A*(1-tauy)/(((1+n)/beta)-(1-delta)) )^(1/(1-alpha));
yss=A*thetass*kss^alpha;
css=(kss/(1+tauc))*((1-tauy)*(yss/kss)-(n+delta));
iss=(n+delta)*kss;
rss=alpha*(yss/kss)-delta;
gss=tauy*yss+tauc*css;
gyss=gss/yss;
ydss=(1-tauy)*yss;


%  initial conditions

k0=kss;         % capital stock 
theta0=thetass; % shock 

Mmean=[];       % matrix of means 
Mstd=[];        % matrix of standard desviations 
Mcv=[];         % matrix of coefficients of variation 
Mcorry=[];      % matrix of correlations with output 
Mcorry1=[];     % matrix of correlations with output(+1) 
Mcorry2=[];     % matrix of correlations with output(+2) 
Mcorrym1=[];    % matrix of correlations with output(-1) 
Mcorrym2=[];    % matrix of correlations with output(-2)
Merror=[];      % matrix of expectations errors 

    datac = [];
    datak = [];
    datay = [];
    datag = [];
    datai = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for j=1:nreal;
    j

    eps=randn(nobs+100,1)*sigmae;
    eps=eps(101:nobs+100); % innovations

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %  Activate the line corresponding to the solution method you want to
    %  use
    %
    %[c,k,theta,y,yd,i,g,gy,cy,iy,ferror]=MBK(eps,theta0,k0,css,kss,thetass,nobs,param);
    %[c,k,theta,y,yd,i,g,gy,cy,iy,ferror]=MLQ(eps,theta0,k0,css,kss,thetass,nobs,param);
    %[c,k,theta,y,yd,i,g,gy,cy,iy,ferror]=MSIMS(eps,theta0,k0,css,kss,thetass,nobs,param);
    %[c,k,theta,y,yd,i,g,gy,cy,iy,ferror]=MLSIMS(eps,theta0,k0,css,kss,thetass,nobs,param);
    %[c,k,theta,y,yd,i,g,gy,cy,iy,ferror]=MUHLIG(eps,theta0,k0,css,kss,thetass,nobs,param);
    load a0.prn; 
    [c,k,theta,y,yd,i,g,gy,cy,iy,ferror]=MMARCET(eps,theta0,k0,css,kss,thetass,nobs,param,a0);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    T=length(ferror);
    c=c(1:T);
    k=k(1:T);
    y=y(1:T);
    g=g(1:T);
    i=i(1:T);
    iy=iy(1:T);
    gy=gy(1:T);
    cy=cy(1:T);
    yd=yd(1:T);
    
    datac = [datac c];
    datak = [datak k];
    datay = [datay y];
    datag = [datag g];
    datai = [datai i];
    
    V=[c k y i g gy cy iy yd];
    
    Mmean=[Mmean; mean(V)];
    Mstd=[Mstd; std(V)];
    Mcv=[Mcv; std(V)./mean(V)];
        pc=corrcoef(c,y);
        pk=corrcoef(k,y);
        pi=corrcoef(i,y);
        pg=corrcoef(g,y);
        pgy=corrcoef(gy,y);
        pcy=corrcoef(cy,y);
        piy=corrcoef(iy,y);
    Mcorry=[Mcorry; pc(1,2) pk(1,2) pi(1,2) pg(1,2) pgy(1,2) pcy(1,2) piy(1,2)];   
        pc=corrcoef(c(1:T-1),y(2:T));
        pk=corrcoef(k(1:T-1),y(2:T));
        pi=corrcoef(i(1:T-1),y(2:T));
        pg=corrcoef(g(1:T-1),y(2:T));
        pgy=corrcoef(gy(1:T-1),y(2:T));
        pcy=corrcoef(cy(1:T-1),y(2:T));
        piy=corrcoef(iy(1:T-1),y(2:T));
    Mcorry1=[Mcorry1; pc(1,2) pk(1,2) pi(1,2) pg(1,2) pgy(1,2) pcy(1,2) piy(1,2)];   
        pc=corrcoef(c(1:T-2),y(3:T));
        pk=corrcoef(k(1:T-2),y(3:T));
        pi=corrcoef(i(1:T-2),y(3:T));
        pg=corrcoef(g(1:T-2),y(3:T));
        pgy=corrcoef(gy(1:T-2),y(3:T));
        pcy=corrcoef(cy(1:T-2),y(3:T));
        piy=corrcoef(iy(1:T-2),y(3:T));
    Mcorry2=[Mcorry2; pc(1,2) pk(1,2) pi(1,2) pg(1,2) pgy(1,2) pcy(1,2) piy(1,2)];   
        pc=corrcoef(c(2:T),y(1:T-1));
        pk=corrcoef(k(2:T),y(1:T-1));
        pi=corrcoef(i(2:T),y(1:T-1));
        pg=corrcoef(g(2:T),y(1:T-1));
        pgy=corrcoef(gy(2:T),y(1:T-1));
        pcy=corrcoef(cy(2:T),y(1:T-1));
        piy=corrcoef(iy(2:T),y(1:T-1));
    Mcorrym1=[Mcorrym1; pc(1,2) pk(1,2) pi(1,2) pg(1,2) pgy(1,2) pcy(1,2) piy(1,2)];   
        pc=corrcoef(c(3:T),y(1:T-2));
        pk=corrcoef(k(3:T),y(1:T-2));
        pi=corrcoef(i(3:T),y(1:T-2));
        pg=corrcoef(g(3:T),y(1:T-2));
        pgy=corrcoef(gy(3:T),y(1:T-2));
        pcy=corrcoef(cy(3:T),y(1:T-2));
        piy=corrcoef(iy(3:T),y(1:T-2));
    Mcorrym2=[Mcorrym2; pc(1,2) pk(1,2) pi(1,2) pg(1,2) pgy(1,2) pcy(1,2) piy(1,2)];   
        merror=mean(ferror);
        stderror=std(ferror);
        perror1=corrcoef(ferror(1:T-1),ferror(2:T));perror1=perror1(1,2);
        perror2=corrcoef(ferror(1:T-2),ferror(3:T));perror2=perror2(1,2);
        perror3=corrcoef(ferror(1:T-3),ferror(4:T));perror3=perror3(1,2);
        perror4=corrcoef(ferror(1:T-4),ferror(5:T));perror4=perror4(1,2);
        perrory=corrcoef(ferror(1:T-1),y(2:T));perrory=perrory(1,2);
        perrorc=corrcoef(ferror(1:T-1),c(2:T));perrorc=perrorc(1,2);
        perrork=corrcoef(ferror(1:T-1),k(2:T));perrork=perrork(1,2);
        perrorg=corrcoef(ferror(1:T-1),g(2:T));perrorg=perrorg(1,2);
        perrori=corrcoef(ferror(1:T-1),i(2:T));perrori=perrori(1,2);
    Merror=[Merror; merror stderror perror1 perror2 perror3 perror4 perrory perrorc perrork perrorg perrori];

end

    AVGm=mean(Mmean);
    AVGs=mean(Mstd);
    AVGcv=mean(Mcv);
    AVGcy=mean(Mcorry);
    AVGcy1=mean(Mcorry1);
    AVGcy2=mean(Mcorry2);
    AVGcym1=mean(Mcorrym1);
    AVGcym2=mean(Mcorrym2);
    AVGerr=mean(Merror);
    
    STDm=std(Mmean)./(nreal^0.5);
    STDs=std(Mstd)./(nreal^0.5);
    STDcv=std(Mcv)./(nreal^0.5);
    STDcy=std(Mcorry)./(nreal^0.5);
    STDcy1=std(Mcorry1)./(nreal^0.5);
    STDcy2=std(Mcorry2)./(nreal^0.5);
    STDcym1=std(Mcorrym1)./(nreal^0.5);
    STDcym2=std(Mcorrym2)./(nreal^0.5);
    STDerr=std(Merror)./(nreal^0.5);

save means.prn AVGm STDm -ascii   
save stds.prn AVGs STDs -ascii
save cvs.prn AVGcv STDcv -ascii
save cys.prn AVGcy STDcy -ascii
save cy1s.prn AVGcy1 STDcy1 -ascii
save cy2s.prn AVGcy2 STDcy2 -ascii
save cym1s.prn AVGcym1 STDcym1 -ascii
save cym2s.prn AVGcym2 STDcym2 -ascii
save errs.prn AVGerr STDerr -ascii

    save datac.prn datac -ascii   
    save datak.prn datak -ascii   
    save datay.prn datay -ascii   
    save datai.prn datai -ascii   
    save datag.prn datag -ascii   

    