% This program computes a single realization from the discrete-time version of the Cass-Koopmans 
% growth model under uncertainty using Uhlig's solution method, based on a log-linear approximation. 
% 
% The program calls function ss_lsims.m, used to solve for the steady-state as well as to 
% compute the transition matrix for the linear approximation to the first order conditions of 
% the optimization problem.
%
% The single Figure displays the time series paths for the endogenous variables
% 
% Structural parameter values can be changed in the first block of instructions.
%
% [Copyright: J. Ruiz]

clear

nobs=500;            % number of observations

%  structural parameters

A=1.0;               %    technology level
beta= 0.95;      	   %    discount factor
sigma=3.0;      	   %    inverse of the intertemporal elasticity of substitution (relative risk aversion)
n=0.01;              %    population growth rate
delta=0.1;           %    depreciation rate
alpha=0.35;          %    output share of capital 
tauc=0.12;      	   %    consumption tax rate
tauy=0.2;          	%    income tax rate
rho=0.95;         	%    autoregressive parameter
sigmae=0.005;     	%    standard desviation of innovation in productivity


%  steady state

thetass0=1; lthetss0=log(thetass0);
kss0=( alpha*thetass0*A*(1-tauy)/(((1+n)/beta)-(1-delta)) )^(1/(1-alpha));
lkss0=log(kss0);
yss0=A*thetass0*kss0^alpha;
css0=(kss0/(1+tauc))*((1-tauy)*(yss0/kss0)-(n+delta));
lcss0=log(css0);
iss0=(n+delta)*kss0;
rss0=alpha*(yss0/kss0)-delta;
gss0=tauy*yss0+tauc*css0;
gyss0=gss0/yss0;
ydss0=(1-tauy)*yss0;


% an alternative approach to calculate the steady state
    param=[A beta sigma n delta alpha tauc tauy rho sigmae];
    x0=[lkss0 lkss0 lcss0 lcss0 lthetss0 lthetss0 0 0]';
    x=fsolve('ss_lsims',x0,optimset,param);
    lkss=x(1);kss=exp(lkss);
    lcss=x(3); css=exp(lcss);
    thetass=exp(x(5));lthetass=x(5);
    ferrss=x(8);
    innss=x(7);
    yss=A*thetass*kss^alpha; 
    iss=(n+delta)*kss; 
    rss=alpha*(yss/kss)-delta;
    uss=(css^(1-sigma)-1)/(1-sigma);      
    gss=tauy*yss+tauc*css; 
    gyss=gss/yss; 
    ydss=(1-tauy)*yss;
    
    J=jacobian('ss_lsims',x,param); % transition matrix
    alpha1=J(1,4);
    alpha2=J(1,1);
    alpha3=J(1,6);
    alpha4=J(1,2);
    beta1=J(2,4);
    beta2=J(2,1);
    beta3=J(2,5);
    beta4=J(2,3);
    etakk1=((alpha1*beta2-beta1*alpha2-beta4*alpha4)+...
        ((alpha1*beta2-beta1*alpha2-beta4*alpha4)^2-4*beta4*alpha2*...
        beta1*alpha4)^0.5)/(2*beta4*alpha2);
    etakk2=((alpha1*beta2-beta1*alpha2-beta4*alpha4)-...
        ((alpha1*beta2-beta1*alpha2-beta4*alpha4)^2-4*beta4*alpha2*...
        beta1*alpha4)^0.5)/(2*beta4*alpha2);
    if abs(etakk1)<1
        etakk=etakk1;
    else
        etakk=etakk2;
    end
    etack=-(alpha2*etakk+alpha4)/alpha1;
    etakt=((beta1+rho*beta4)*alpha3-alpha1*beta3*rho)/...
        (beta2*alpha1+beta4*alpha1*etack-(beta1+rho*beta4)*alpha2);
    etact=-(alpha3+alpha2*etakt)/alpha1;
    
   
%  initial conditions

k0=kss;         				% capital stock 
theta0=thetass; 				% shock 

eps=randn(nobs+100,1)*sigmae;
eps=eps(101:nobs+100); 		% innovations

lc0=lcss+etack*(log(k0)-lkss)+etact*(log(theta0));
c0=exp(lc0);
c=[];c=[c; c0];
k=[];k=[k; k0];
theta=[];theta=[theta; theta0];
y=[];y0=theta0*A*(k0^alpha);y=[y;y0];
i=[];  
r=[];r0=alpha*(y0/k0)-delta; r=[r;r0];
g=[];g0=tauy*y0+tauc*c0; g=[g;g0];
gy=[]; gy0=g0/y0; gy=[gy;gy0];
cy=[]; cy0=c0/y0; cy=[cy;cy0];
iy=[];
yd=[]; yd0=(1-tauy)*y0; yd=[yd;yd0];
ferror=[];


for t=1:nobs

    lk1=lkss+etakk*(log(k0)-lkss)+etakt*log(theta0);k1=exp(lk1);
    theta1=exp(rho*log(theta0)+eps(t));
    lc1=lcss+etack*(log(k1)-lkss)+etact*(log(theta1));
    c1=exp(lc1);
    y1=theta1*A*(k1^alpha);y=[y;y1];
    i1=(1+n)*k1-(1-delta)*k0;i=[i;i1];
    iy1=i1/y1; iy=[iy;iy1];
    ferror1=(c1^(-sigma))*((1-tauy)*alpha*A*theta1*...
          (k1^(alpha-1))+(1-delta))-((1+n)/beta)*(c0^(-sigma));
    ferror=[ferror;ferror1];
        
    r1=alpha*(y1/k1)-delta; r=[r;r1];
    g1=tauy*y1+tauc*c1; g=[g;g1];
    gy1=g1/y1; gy=[gy;gy1];
    cy1=c1/y1; cy=[cy;cy1];
    yd1=(1-tauy)*y1; yd=[yd;yd1];
    k0=k1;c0=c1;theta0=theta1;
    k=[k;k0];c=[c;c0];theta=[theta;theta0];
    
end
    
   
    
% graphs
vcss=ones(size(c,1),1)*css;
vyss=ones(size(c,1),1)*yss;
vkss=ones(size(c,1),1)*kss;
vrss=ones(size(c,1),1)*rss;
vthss=ones(size(c,1),1)*thetass;
viss=ones(size(c,1)-1,1)*iss;
vgss=ones(size(c,1),1)*gss;
vgyss=ones(size(c,1),1)*gyss;
vydss=ones(size(c,1),1)*ydss;


figure;
hold on

subplot(4,2,1)
plot([c,vcss]);
title('consumption');
xlabel('time');

subplot(4,2,2)
plot([y,vyss]);
title('output');
xlabel('time');

subplot(4,2,3)
plot([k,vkss]);
title('capital stock');
xlabel('time');

subplot(4,2,4)
plot([i,viss]);
title('investment');
xlabel('time');

subplot(4,2,5)
plot([g,vgss]);
title('public revenues');
xlabel('time');

subplot(4,2,6)
plot([gy,vgyss]);
title('public revenues to output ratio');
xlabel('time');

subplot(4,2,7)
plot([yd,vydss]);
title('disposable income');
xlabel('time');


hold off